/*
* Copyright (c) 2005 Nokia Corporation and/or its subsidiary(-ies). 
* All rights reserved.
* This component and the accompanying materials are made available
* under the terms of "Eclipse Public License v1.0"
* which accompanies this distribution, and is available
* at the URL "http://www.eclipse.org/legal/epl-v10.html".
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description: Operation for parsing a XML file with Landmark Content Format.
*
*
*/


#ifndef CPOSLMXMLPARSEROPERATION_H
#define CPOSLMXMLPARSEROPERATION_H

//  INCLUDES
#include "EPos_CPosLmConverterOperation.h"

// CONSTANTS

// FORWARD DECLARATIONS
class CPosLmXmlParser;

// CLASS DECLARATION

/**
* Operation for parsing a XML file with Landmark Content Format.
* This operation can be either synchronous or asynchronous.
*
*  @lib eposconverter.lib
*/
class CPosLmXmlParserOperation : public CPosLmConverterOperation
    {

    public:  // Constructors and destructor

        /**
        * Two-phased constructor.
        * @param aParser A Landmark XML parser.
        * return Operation for operating on Landmark XML parser.
        */
        static CPosLmXmlParserOperation* NewL(
        /* IN */        CPosLmXmlParser& aParser
        );

        /**
        * Destructor.
        */
        virtual ~CPosLmXmlParserOperation();

    public: // New functions

        /**
        * Notify operation that parser is destroyed.
        */
        void DisconnectParser();

    public: // Functions from base classes

        /**
        * From CPosLmConverterOperation
        *
        * @param aProgress Progress of the operation.
        * @return KErrNone if parsing is ready, otherwise
        * KPosLmOperationNotComplete.
        */
        TInt NextStepL(
        /* OUT */   TReal32& aProgress
        );

        /**
        * From CPosLmConverterOperation.
        *
        * @param aError An error code generated by NextStepL.
        */
        void HandleError(
        /* IN/OUT */    TInt& aError
        );

    private:

        /**
        * C++ default constructor.
        * @param aParser A Landmark XML parser.
        */
        CPosLmXmlParserOperation(
        /* IN */        CPosLmXmlParser& aLmXmlParser
        );

        /**
        * By default Symbian 2nd phase constructor is private.
        */
        void ConstructL();

        // Prohibit copy constructor if not deriving from CBase.
        CPosLmXmlParserOperation(const CPosLmXmlParserOperation&);
        // Prohibit assigment operator if not deriving from CBase.
        CPosLmXmlParserOperation& operator=(const CPosLmXmlParserOperation&);


    private:    // Data

        // Landmark Xml Parser.
        CPosLmXmlParser*    iLmXmlParser;

        // Status flag. Holding the current status.
        TInt iStatusFlag;

    };

#endif      // CPOSLMXMLPARSEROPERATION_H

// End of File
